package com.palmergames;

import org.jibble.pircbot.*;
import org.jibble.jmegahal.*;

public class Mypirc extends PircBot {

	private String channel = "#Towny,#Towny-Dev";
	private JMegaHal hal = new JMegaHal();

	private String[] connectors = { "then", "what", "how", "that", "they", "the", "and", "you", "'", "are", "can" };

	public void setHal(Object jMegaHal) {

		this.hal = (JMegaHal) jMegaHal;
	}

	public JMegaHal getHal() {

		return hal;
	}

	/**
	 * Constructor
	 * 
	 * @param name
	 */
	public Mypirc(String name) {

		this.setName(name);
	}

	public void start(String[] args) throws Exception {

		// Enable debugging output.
		this.setVerbose(true);

		// Connect to the IRC server.
		this.connect("irc.esper.net");
		this.identify("testing01");

		// Join all channels.
		for (String match : channel.split(","))
			this.joinChannel(match);


	}

	@Override
	public void onMessage(String channel, String sender, String login, String hostname, String message) {

		message = Colors.removeFormattingAndColors(message);

		HenryUI.textArea.append(sender + ": " + message + System.getProperty("line.separator"));

		if (message.equalsIgnoreCase("!time")) {
			
			// The time was asked or.
			String time = new java.util.Date().toString();
			sendMessage(channel, sender + ": The time is now " + time);
			
		} else if (message.equalsIgnoreCase("!talk") && (isSenderOp(channel, sender))) {
			
			// Someone commanded us to talk.
			sendMessage(channel, sender + ", " + hal.getSentence());

		} else if (message.toLowerCase().indexOf("?") >= 0) {
			
			// If a question was asked, try to find a reply.
			message = fetchSentence(message);
			if (message != null)
				sendMessage(channel, sender + ", " + message);
			
		} else if (message.toLowerCase().startsWith("!learn ") && (isSenderOp(channel, sender))){

			message = message.replaceFirst("!learn ", "");
			// Learn this response
			if (hal.add(message)) {
				HenryUI.textArea.append("Learning : " + message + System.getProperty("line.separator"));
				// Save this message as we learned it.
				HenryUI.saveSentence(message);
			} else
				HenryUI.textArea.append("We were unable to learn anything from this." + System.getProperty("line.separator"));

		}
	}
	
	public boolean isSenderOp(String channel, String sender) {
		
		for (User user : this.getUsers(channel))
			if (user.getNick().equalsIgnoreCase(sender) && user.isOp())
				return true;
			
		return false;
	}

	private String fetchSentence(String message) {

		// Remove all insignificant words.
		String result = message.toLowerCase();
		for (String match : connectors) {

			result = result.replaceAll("\\b" + match + "\\b", "");
		}
		
		// Also remove our nick from the keywords.
		result = result.replaceAll("(?i)" + getNick().toLowerCase() + "", "");

		// Remove all punctuation.
		//result = result.replaceAll("[^a-zA-Z0-9._-]", " ");

		//Split on punctuation and spaces.
		String[] words = result.split("[\\p{P} \\s+]", 0);
		int weight = 0;

		for (String match : words) {
			if (!match.isEmpty() && (match.trim().length() > 2)) {
				String sentence = hal.getSentence(match.trim());

				int thisWeight = getWeight(sentence, words);

				if (thisWeight > weight) {
					weight = thisWeight;
					result = sentence;
				}
			}
		}

		// Return null if we can't find a suitable response.
		return (weight == 0)? null : result;
	}

	private int getWeight(String message, String[] words) {

		int result = 0;
		message = message.toLowerCase();
		
		for (String match : words) {
			if (!match.isEmpty() && (match.trim().length() > 2)) {

				String[] split = message.split("\\b" + match.trim().toLowerCase() + "\\b", 0);

				HenryUI.textArea.append("    Testing: " + match + " - against : " + message + System.getProperty("line.separator"));

				if (split.length > 1)
					result++;
			}
		}
		return result;
	}

	public String getChannel() {

		return channel.split(",")[0];
	}

	public void setChannel(String channel) {

		this.channel = channel;
	}

}